@@ -25,6 +25,8 @@
 #' @param params List of parameters; Add columns with information to returning data.frame
 #' @param verbose Logical; To show more information
 #' @param fortran Logical; to try the fortran calculation.
+#' @param nt Integer; Number of threads wich must be lower than max available. See \code{\link{check_nt}}.
+#' Only when fortran = TRUE
 #' @return Emissions data.frame
 #' @seealso \code{\link{ef_ldv_cold}}
 #' @export
@@ -99,7 +101,8 @@ emis_cold_td <- function (veh,
                           pro_month,
                           params,
                           verbose = FALSE,
-                          fortran = FALSE) {
+                          fortran = FALSE,
+                          nt = ifelse(check_nt()==1, 1, check_nt()/2)) {
   # Check units
   if(class(lkm) != "units"){
     stop("lkm neeeds to has class 'units' in 'km'. Please, check package '?units::set_units'")
@@ -117,10 +120,10 @@ emis_cold_td <- function (veh,
     if(class(ef[, 1]) != "units"){
       stop("columns of ef must has class 'units' in 'g/km'. Please, check package '?units::set_units'")
     }
-    if(units(ef[, 1])$numerator != "g" | units(ef[, 1])$denominator != "km"){
+    if(units(ef[, 1])$numerator != "g" || units(ef[, 1])$denominator != "km"){
       stop("Units of efcold must be 'g/km' ")
     }
-    if(units(ef[, 1])$numerator == "g" | units(ef[, 1])$denominator == "km"){
+    if(units(ef[, 1])$numerator == "g" || units(ef[, 1])$denominator == "km"){
       for(i in 1:ncol(veh)){
         ef[, i] <- as.numeric(ef[, i])
       }
@@ -129,22 +132,22 @@ emis_cold_td <- function (veh,
     if(class(ef) != "units"){
       stop("ef must has class 'units' in 'g/km'. Please, check package '?units::set_units'")
     }
-    if(units(ef)$numerator != "g" | units(ef)$denominator != "km"){
+    if(units(ef)$numerator != "g" || units(ef)$denominator != "km"){
       stop("Units of ef must be 'g/km' ")
     }
-    if(units(ef)$numerator == "g" | units(ef)$denominator == "km"){
-      ef <- as.numeric(ef)
-    }
+    # if(units(ef)$numerator == "g" || units(ef)$denominator == "km"){ # not tested
+      # ef <- as.numeric(ef)
+    # }
   }
 
   # Checking ef cold
   if(class(efcold[, 1]) != "units"){
     stop("columns of efcold must has class 'units' in 'g/km'. Please, check package '?units::set_units'")
   }
-  if(units(efcold[, 1])$numerator != "g" | units(efcold[, 1])$denominator != "km"){
+  if(units(efcold[, 1])$numerator != "g" || units(efcold[, 1])$denominator != "km"){
     stop("Units of efcold must be 'g/km' ")
   }
-  if(units(efcold[, 1])$numerator == "g" & units(efcold[, 1])$denominator == "km"){
+  if(units(efcold[, 1])$numerator == "g" && units(efcold[, 1])$denominator == "km"){
     for(i in 1:ncol(veh)){
       efcold[, i] <- as.numeric(efcold[, i])
     }
@@ -188,34 +191,72 @@ emis_cold_td <- function (veh,
     if(is.data.frame(ef)){
       if(verbose) message("Assuming you have emission factors for each simple feature and then for each month")
 
-
       #when pro_month varies in each simple feature
       if(is.data.frame(pro_month)){
+
+        if(nrow(pro_month) == 1) {
+          message("Replicating one-row matrix to match number of rows of `veh`")
+          pro_month <- matrix(as.numeric(pro_month), nrow = nrow(veh), ncol = ncol(pro_month))
+        }
+
         if(fortran){
+          efcold$month <- rep(1:12, each = nrow(veh))
+          efcold <- split(efcold[, 1:ncol(veh)], efcold$month)
+          efcold <- as.numeric(unlist(lapply(efcold, unlist)))
+
           nrowv <- as.integer(nrow(veh))
           ncolv <- as.integer(ncol(veh))
           pmonth <- as.integer(ncol(pro_month))
           lkm <- as.numeric(lkm)
           ef <- as.matrix(ef)
-          if(nrow(ef) != nrow(veh)) stop("rows of 'ef' and 'veh' must be equal")
-          efcold <- split(efcold[, 1:ncol(veh)], efcold[ncol(efcold)])
-          efcold <- as.numeric(unlist(lapply(efcold, unlist)))
+
           month <- as.matrix(pro_month)
           beta <- as.matrix(beta)
 
-          if(verbose) message("Calling emistd4coldf.f95")
+          if(nrow(beta) != nrow(veh)) stop("number of rows of 'beta' and 'veh' must be equal")
+          if(ncol(beta) != ncol(month)) stop("number of cols of 'beta' and 'month' must be equal")
+          if(length(efcold) != length(unlist(veh))*pmonth) stop("`efcold` and `veh` must be dimensionally compatible")
+          if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+          if(nrow(ef) != nrow(veh)) stop("number of rows of `ef` and `veh` must be equal")
+          if(ncol(ef) != ncol(veh)) stop("number of cols of `ef` and `veh` must be equal")
+          if(nrow(month) != nrow(veh)) stop("number of rows of `month` and `veh` must be equal")
+          # emis(i, j, k) = beta(i, k) * veh(i, j) * lkm(j) * ef(i, j) * efcold(i, j, k) * month(i, k)
 
-          a <-   .Fortran("emistd4coldf",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          ef = efcold,
-                          beta = beta,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd4coldfpar.f95")
+
+            a <-   .Fortran("emistd4coldfpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            efcold = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+
+          } else {
+            if(verbose) message("Calling emistd4coldf.f95")
+
+            a <-   .Fortran("emistd4coldf",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            efcold = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -247,26 +288,56 @@ emis_cold_td <- function (veh,
           pmonth <- as.integer(length(pro_month))
           lkm <- as.numeric(lkm)
           ef <- as.matrix(ef[, 1:ncol(veh)])
-          if(nrow(ef) != nrow(veh)) stop("rows of 'ef' and 'veh' must be equal")
           efcold$month <- rep(1:12, each = nrow(veh))
           efcold <- split(efcold[, 1:ncol(veh)], efcold$month)
           efcold <- as.numeric(unlist(lapply(efcold, unlist)))
           month <- as.numeric(pro_month)
           beta <- as.matrix(beta)
 
-          if(verbose) message("Calling emistd5coldf.f95")
 
-          a <-   .Fortran("emistd5coldf",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          ef = efcold,
-                          beta = beta,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(nrow(beta) != nrow(veh)) stop("number of rows of 'beta' and 'veh' must be equal")
+          if(ncol(beta) != length(month)) stop("number of cols of 'beta' length of 'month' must be equal")
+          if(length(efcold) != length(unlist(veh))*pmonth) stop("`efcold` and `veh` must be dimensionally compatible")
+          if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+          if(nrow(ef) != nrow(veh)) stop("number of rows of `ef` and `veh` must be equal")
+          if(ncol(ef) != ncol(veh)) stop("number of cols of `ef` and `veh` must be equal")
+          # emis(i, j, k) = beta(i, k) * veh(i, j) * lkm(j) * ef(i, j) * efcold(i, j, k) * month(k)
+
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd5coldfpar.f95")
+
+            a <-   .Fortran("emistd5coldfpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            ef = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+
+          } else {
+            if(verbose) message("Calling emistd5coldf.f95")
+
+            a <-   .Fortran("emistd5coldf",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            ef = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
           e <- data.frame(emissions = a)
           e <- Emissions(e)
           e$rows <- rep(row.names(veh), ncolv*pmonth)
@@ -302,12 +373,20 @@ emis_cold_td <- function (veh,
         }
       }
       if(verbose) cat("Sum of emissions:", sum(e$emissions), "\n")
-    } else{
+
+    } else {
+      #ef inherits numeric
       if(verbose) message("Assuming you have emission factors for each simple feature and then for each month")
 
 
-      # when pro_month variy each month
+      # when pro_month vary each month
       if(is.data.frame(pro_month)){
+
+        if(nrow(pro_month) == 1) {
+          message("Replicating one-row matrix to match number of rows of `veh`")
+          pro_month <- matrix(as.numeric(pro_month), nrow = nrow(veh), ncol = ncol(pro_month))
+        }
+
         if(fortran){
           nrowv <- as.integer(nrow(veh))
           ncolv <- as.integer(ncol(veh))
@@ -320,20 +399,49 @@ emis_cold_td <- function (veh,
           month <- as.matrix(pro_month)
           beta <- as.matrix(beta)
 
-          if(verbose) message("Calling emistd6coldf.f95")
 
-          a <-   .Fortran("emistd6coldf",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          ef = efcold,
-                          beta = beta,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(nrow(beta) != nrow(veh)) stop("number of rows of 'beta' and 'veh' must be equal")
+          if(ncol(beta) != ncol(month)) stop("number of cols of 'beta' and 'month' must be equal")
+          if(length(efcold) != length(unlist(veh))*pmonth) stop("`efcold` and `veh` must be dimensionally compatible")
+          if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+          if(length(ef) != ncol(veh)) stop("length of `ef` and number of cols of `veh` must be equal")
+          if(nrow(month) != nrow(veh)) stop("number of rows of `month` and `veh` must be equal")
+          # emis(i, j, k) = beta(i, k) * veh(i, j) * lkm(j) * ef(j) * efcold(i, j, k) * month(i, k)
 
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd6coldfpar.f95")
+
+            a <-   .Fortran("emistd6coldfpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            ef = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+          } else {
+            if(verbose) message("Calling emistd6coldf.f95")
+
+            a <-   .Fortran("emistd6coldf",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            ef = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
           e <- data.frame(emissions = a)
           e <- Emissions(e)
           e$rows <- rep(row.names(veh), ncolv*pmonth)
@@ -369,19 +477,47 @@ emis_cold_td <- function (veh,
           month <- as.numeric(pro_month)
           beta <- as.matrix(beta)
 
-          if(verbose) message("Calling emistd3coldf.f95")
+          if(nrow(beta) != nrow(veh)) stop("number of rows of 'beta' and 'veh' must be equal")
+          if(ncol(beta) != length(month)) stop("number of cols of 'beta' length of 'month' must be equal")
+          if(length(efcold) != length(unlist(veh))*pmonth) stop("`efcold` and `veh` must be dimensionally compatible")
+          if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+          if(length(ef) != ncol(veh)) stop("number of rows of `ef` and `veh` must be equal")
+          # emis(i, j, k) = beta(i, k) * veh(i, j) * lkm(j) * ef(j) * efcold(i, j, k) * month(k)
 
-          a <-   .Fortran("emistd3coldf",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          ef = efcold,
-                          beta = beta,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd3coldfpar.f95")
+
+            a <-   .Fortran("emistd3coldfpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            ef = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+          } else {
+            if(verbose) message("Calling emistd3coldf.f95")
+
+            a <-   .Fortran("emistd3coldf",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            ef = efcold,
+                            beta = beta,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -427,17 +563,45 @@ emis_cold_td <- function (veh,
       efcold <- as.matrix(efcold[, 1:ncol(veh)])
       beta <- as.numeric(unlist(beta))
 
-      if(verbose) message("Calling emistd2coldf.f95")
 
-      a <-   .Fortran("emistd2coldf",
-                      nrowv = nrowv,
-                      ncolv = ncolv,
-                      veh = as.matrix(veh),
-                      lkm = lkm,
-                      ef = ef,
-                      ef = efcold,
-                      beta = beta,
-                      emis = numeric(nrowv*ncolv))$emis
+      if(length(beta) != nrow(veh)) stop("length of 'beta' and number of rows of 'veh' must be equal")
+      if(nrow(efcold) != nrow(veh)) stop("number of rows of 'efcold' and 'veh' must be equal")
+      if(ncol(efcold) != ncol(veh)) stop("number of cols of 'efcold' and 'veh' must be equal")
+      if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+      if(length(ef) != ncol(veh)) stop("number of rows of `ef` and `veh` must be equal")
+      # emis(i, j) = beta(i) * veh(i, j) * lkm(j) * ef(j) * efcold(i, j)
+
+
+      if(!missing(nt)) {
+        if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                  " threads and nt must be lower")
+
+        if(verbose) message("Calling emistd2coldfpar.f95")
+
+        a <-   .Fortran("emistd2coldfpar",
+                        nrowv = nrowv,
+                        ncolv = ncolv,
+                        veh = as.matrix(veh),
+                        lkm = lkm,
+                        ef = ef,
+                        ef = efcold,
+                        beta = beta,
+                        emis = numeric(nrowv*ncolv),
+                        nt = as.integer(nt))$emis
+      } else {
+        if(verbose) message("Calling emistd2coldf.f95")
+
+        a <-   .Fortran("emistd2coldf",
+                        nrowv = nrowv,
+                        ncolv = ncolv,
+                        veh = as.matrix(veh),
+                        lkm = lkm,
+                        ef = ef,
+                        ef = efcold,
+                        beta = beta,
+                        emis = numeric(nrowv*ncolv))$emis
+
+      }
 
       e <- data.frame(emissions = a)
       e <- Emissions(e)
