@@ -24,9 +24,23 @@
 #' @param params List of parameters; Add columns with information to returning data.frame
 #' @param verbose Logical; To show more information
 #' @param fortran Logical; to try the fortran calculation.
+#' @param nt Integer; Number of threads wich must be lower than max available. See \code{\link{check_nt}}.
+#' Only when fortran = TRUE
 #' @return Emissions data.frame
 #' @seealso \code{\link{ef_ldv_speed}} \code{\link{ef_china}}
 #' @export
+#' @details List  to make easier to use this function.
+#' \enumerate{
+#'  \item{`pro_month` is data.frame AND rows of `ef` and `veh` are equal.}
+#'  \item{`pro_month` is numeric AND rows of `ef` and `veh` are equal.}
+#'  \item{`pro_month` is data.frame AND rows of `ef` is 12X rows of `veh`.}
+#'  \item{`pro_month` is numeric AND rows of `ef` is 12X rows of `veh`.}
+#'  \item{`pro_month` is data,frame AND class of `ef` is 'units'.}
+#'  \item{`pro_month` is numeric AND class of `ef` is 'units'.}
+#'  \item{NO `pro_month` AND class of `ef` is 'units'.}
+#'  \item{NO `pro_month` AND `ef` is data.frame.}
+#'  \item{`pro_month` is numeric AND rows of `ef` is 12 (monthly `ef`).}
+#' }
 #' @examples \dontrun{
 #' # Do not run
 #' euros <- c("V", "V", "IV", "III", "II", "I", "PRE", "PRE")
@@ -175,7 +189,8 @@ emis_hot_td <- function (veh,
                          pro_month,
                          params,
                          verbose = FALSE,
-                         fortran = FALSE) {
+                         fortran = FALSE,
+                         nt = ifelse(check_nt()==1, 1, check_nt()/2)) {
   # Checking sf
   if(any(class(veh) %in% "sf")){
     if(verbose) message("converting sf to data.frame")
@@ -255,7 +270,16 @@ emis_hot_td <- function (veh,
 
         if(verbose) message("'pro_month' is data.frame and number of rows of 'ef' and 'veh' are equal")
 
+
+        if(nrow(pro_month) == 1) {
+          message("Replicating one-row matrix to match number of rows of `veh`")
+          pro_month <- matrix(as.numeric(pro_month), nrow = nrow(veh), ncol = ncol(pro_month))
+        }
+
         if(nrow(ef) != nrow(veh)) stop("Number of rows of 'veh' and 'ef' must be equal")
+        if(ncol(ef) != ncol(veh)) stop("Number of cols of `ef` and `veh` must be equal")
+        if(length(lkm) != ncol(veh)) stop("Length of `lkm` must be equal to number of columns of `veh`")
+        if(nrow(pro_month) != nrow(veh)) stop("Number of rows of `month` and `veh` must be equal")
 
         if(fortran){
           nrowv <- as.integer(nrow(veh))
@@ -264,18 +288,36 @@ emis_hot_td <- function (veh,
           lkm <- as.numeric(lkm)
           ef <- as.matrix(ef[, 1:ncol(veh)])
           month <- as.matrix(pro_month)
+          # emis(i, j, k) = veh(i, j) * lkm(j) * ef(i, j)*month(i, k)
 
-          if(verbose) message("Calling emistd2f.f95")
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+            if(verbose) message("Calling emistd2fpar.f95")
+            a <-   .Fortran("emistd2fpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
 
-          a <-   .Fortran("emistd2f",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          } else {
+            if(verbose) message("Calling emistd2f.f95")
+            a <-   .Fortran("emistd2f",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -286,6 +328,7 @@ emis_hot_td <- function (veh,
         } else {
           e <- do.call("rbind",lapply(1:12, function(k){
             dfi <- unlist(lapply(1:ncol(veh), function(j){
+
               lkm[j] * veh[, j] * pro_month[, k] *ef[, j]
             }))
             dfi <- as.data.frame(dfi)
@@ -297,12 +340,14 @@ emis_hot_td <- function (veh,
             dfi
           }))
         }
-
+        # is.numeric(pro_month) & nrow(ef) == nrow(veh) ####
       } else if(is.numeric(pro_month) & nrow(ef) == nrow(veh)){
 
         if(verbose) message("'pro_month' is numeric and number of rows of 'ef' and 'veh' are equal")
 
-        if(nrow(ef) != nrow(veh)) stop("Number of rows of 'veh' and 'ef' must be equal")
+        if(nrow(ef) != nrow(veh)) stop("Number of rows of `ef` and `veh` must be equal")
+        if(ncol(ef) != ncol(veh)) stop("Number of cols of `ef` and `veh` must be equal")
+        if(length(lkm) != ncol(veh)) stop("Length of `lkm` must be equal to number of columns of `veh`")
 
         if(fortran) {
           nrowv <- as.integer(nrow(veh))
@@ -312,17 +357,37 @@ emis_hot_td <- function (veh,
           ef <- as.matrix(ef[, 1:ncol(veh)])
           month <- as.numeric(pro_month)
 
-          if(verbose) message("Calling emistd1f.f95")
+          # emis(i, j, k) = veh(i, j) * lkm(j) * ef(i, j)*month(k)
+
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+            if(verbose) message("Calling emistd1fpar.f95")
+            a <-   .Fortran("emistd1fpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+
+          } else {
+            if(verbose) message("Calling emistd1f.f95")
+            a <-   .Fortran("emistd1f",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
 
-          a <-   .Fortran("emistd1f",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -345,10 +410,63 @@ emis_hot_td <- function (veh,
           }))
         }
 
+        # is.numeric(pro_month) & nrow(ef) == 12 ####
+      } else if(is.numeric(pro_month) & nrow(ef) == 12){
+
+        if(verbose) message("'pro_month' is numeric and you have 12 montly ef")
+
+        # if(fortran) {
+        #   nrowv <- as.integer(nrow(veh))
+        #   ncolv <- as.integer(ncol(veh))
+        #   pmonth <- as.integer(length(pro_month))
+        #   lkm <- as.numeric(lkm)
+        #   ef <- as.matrix(ef[, 1:ncol(veh)])
+        #   month <- as.numeric(pro_month)
+        #
+        #   if(verbose) message("Calling emistd1f.f95")
+        #
+        #   a <-   .Fortran("emistd1f",
+        #                   nrowv = nrowv,
+        #                   ncolv = ncolv,
+        #                   pmonth = pmonth,
+        #                   veh = as.matrix(veh),
+        #                   lkm = lkm,
+        #                   ef = ef,
+        #                   month = month,
+        #                   emis = numeric(nrowv*ncolv*pmonth))$emis
+        #
+        #   e <- data.frame(emissions = a)
+        #   e <- Emissions(e)
+        #   e$rows <- rep(row.names(veh), ncolv*pmonth)            # i
+        #   e$age <- rep(rep(seq(1, ncolv), each = nrowv), pmonth) # j
+        #   e$month <- rep(seq(1, pmonth), each = ncolv*nrowv)            # k
+
+        # } else {
+        e <- do.call("rbind",lapply(1:12, function(k){
+          dfi <- unlist(lapply(1:ncol(veh), function(j){
+            lkm[j] * veh[, j] * pro_month[k] *ef[k, j]
+          }))
+          dfi <- as.data.frame(dfi)
+          names(dfi) <- "emissions"
+          dfi <- Emissions(dfi)
+          dfi$rows <- row.names(veh)
+          dfi$age <- rep(1:ncol(veh), each = nrow(veh))
+          dfi$month <- (1:length(pro_month))[k]
+          dfi
+        }))
+        # }
+
+
+        # is.data.frame(pro_month) & nrow(ef) == 12*nrow(veh) ####
       } else if(is.data.frame(pro_month) & nrow(ef) == 12*nrow(veh)){
 
         if(verbose) message("'pro_month' is data.frame and number of rows of 'ef' is 12*number of rows 'veh'")
 
+        if(nrow(pro_month) != nrow(veh)) stop("Number of rows of 'pmonth' and 'veh' must be equal")
+        if(length(ef2) != length(unlist(veh))*pmonth) stop("length of `ef` and `veh`*`months` must be equal be equal")
+        if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+
+
         if(fortran){
           nrowv <- as.integer(nrow(veh))
           ncolv <- as.integer(ncol(veh))
@@ -359,17 +477,38 @@ emis_hot_td <- function (veh,
           ef2 <- as.numeric(unlist(lapply(ef2, unlist)))
           month <- as.matrix(pro_month)
 
-          if(verbose) message("Calling emistd4f.f95")
+          # emis(i, j, k) = veh(i, j) * lkm(j) * ef(i, j, k) * month(i, k)
+
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd4fpar.f95")
+            a <-   .Fortran("emistd4fpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef2,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+          } else {
+
+            if(verbose) message("Calling emistd4f.f95")
+
+            a <-   .Fortran("emistd4f",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef2,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+          }
 
-          a <-   .Fortran("emistd4f",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef2,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -395,32 +534,58 @@ emis_hot_td <- function (veh,
           }))
 
         }
-
+        # is.numeric(pro_month) & nrow(ef) == 12*nrow(veh) ####
       } else if(is.numeric(pro_month) & nrow(ef) == 12*nrow(veh)){
 
         if(verbose) message("'pro_month' is numeric and number of rows of 'ef' is 12*number of rows 'veh'")
 
         if(fortran){
+
           nrowv <- as.integer(nrow(veh))
           ncolv <- as.integer(ncol(veh))
           pmonth <- as.integer(length(pro_month))
           ef$month <- rep(1:12, each = nrow(veh))
           ef2 <- split(ef[, 1:ncol(veh)], ef$month)
           ef2 <- as.numeric(unlist(lapply(ef2, unlist)))
-         lkm <- as.numeric(lkm)
+          lkm <- as.numeric(lkm)
           month <- as.numeric(pro_month)
 
-          if(verbose) message("Calling emistd3f.f95")
 
-          a <-   .Fortran("emistd3f",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef2,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(length(ef2) != length(unlist(veh))*pmonth) stop("length of `ef` and `veh`*`months` must be equal be equal")
+          if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+          # emis(i, j, k) = veh(i, j) * lkm(j) * ef(i, j, k) * month(k)
+
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+
+            if(verbose) message("Calling emistd3fpar.f95")
+
+            a <-   .Fortran("emistd3fpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef2,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+          } else {
+            if(verbose) message("Calling emistd3f.f95")
+
+            a <-   .Fortran("emistd3f",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef2,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
           e <- data.frame(emissions = a)
           e <- Emissions(e)
           e$rows <- rep(row.names(veh),  ncolv*pmonth)           # i
@@ -444,6 +609,8 @@ emis_hot_td <- function (veh,
             dfi
           }))
         }
+      } else {
+        stop("Condition is not met. Review your input data dn read the documentation, please")
       }
       # else if(nrow(ef) !=  nrow(veh) & nrow(ef) != 12)(
       # stop("The number of rows can be equal to number of rows of veh, or number of rows of veh times 12")
@@ -467,13 +634,20 @@ emis_hot_td <- function (veh,
       if(verbose) message("Assuming you have the same emission factors in each simple feature")
 
       # when pro_month vary each row
+      # is.data.frame(pro_month) | is.matrix(pro_month) ####
       if(is.data.frame(pro_month) | is.matrix(pro_month)){
 
         if(verbose) message("'pro_month' is data.frame and 'ef' is numeric")
 
-        if(length(ef) != ncol(veh)) stop("Number of columns of 'veh' and length of 'ef' must be equal")
 
-        if(length(lkm) != ncol(veh)) stop("Length of 'lkm' and number of columns 'veh' must be equal")
+        if(nrow(pro_month) == 1) {
+          message("Replicating one-row matrix to match number of rows of `veh`")
+          pro_month <- matrix(as.numeric(pro_month), nrow = nrow(veh), ncol = ncol(pro_month))
+        }
+
+        if(length(ef) != ncol(veh)) stop("Length of `ef` and number of cols of `veh` must be equal")
+        if(length(lkm) != ncol(veh)) stop("Length of `lkm` must be equal to number of columns of `veh`")
+        if(nrow(pro_month) != nrow(veh)) stop("Number of rows of `month` and `veh` must be equal")
 
         if(fortran){
           nrowv <- as.integer(nrow(veh))
@@ -483,17 +657,39 @@ emis_hot_td <- function (veh,
           ef <- as.numeric(ef)
           month <- as.matrix(pro_month)
 
-          if(verbose) message("Calling emistd6f.f95")
+          # emis(i, j, k) = veh(i,j) * lkm(j) * ef(j) * month(i, k)
 
-          a <-   .Fortran("emistd6f",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd6fpar.f95")
+
+            a <-   .Fortran("emistd6fpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+
+          } else {
+            if(verbose) message("Calling emistd6f.f95")
+
+            a <-   .Fortran("emistd6f",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -502,6 +698,7 @@ emis_hot_td <- function (veh,
           e$month <- rep(seq(1, pmonth), each = ncolv*nrowv)          # k
 
         } else {
+
           e <- do.call("rbind",lapply(1:12, function(k){
             dfi <- unlist(lapply(1:ncol(veh), function(j){
               lkm[j] * veh[, j] * pro_month[, k] *ef[j]
@@ -516,7 +713,7 @@ emis_hot_td <- function (veh,
           }))
 
         }
-
+        # is.numeric(pro_month) ####
       } else if(is.numeric(pro_month)){
 
         if(verbose) message("'pro_month' is numeric and 'ef' is numeric")
@@ -531,17 +728,42 @@ emis_hot_td <- function (veh,
           month <- as.numeric(pro_month)
           ef <- as.numeric(ef)
 
-          if(verbose) message("Calling emistd5f.f95")
 
-          a <-   .Fortran("emistd5f",
-                          nrowv = nrowv,
-                          ncolv = ncolv,
-                          pmonth = pmonth,
-                          veh = as.matrix(veh),
-                          lkm = lkm,
-                          ef = ef,
-                          month = month,
-                          emis = numeric(nrowv*ncolv*pmonth))$emis
+          if(length(ef) != ncol(veh)) stop("length of `ef` and number of cols of `veh` must be equal")
+          if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+          # emis(i, j, k) = veh(i, j) * lkm(j) * ef(j) * month(k)
+
+          if(!missing(nt)) {
+            if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                      " threads and nt must be lower")
+
+            if(verbose) message("Calling emistd5fpar.f95")
+
+            a <-   .Fortran("emistd5fpar",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth),
+                            nt = as.integer(nt))$emis
+
+          } else {
+            if(verbose) message("Calling emistd5f.f95")
+
+            a <-   .Fortran("emistd5f",
+                            nrowv = nrowv,
+                            ncolv = ncolv,
+                            pmonth = pmonth,
+                            veh = as.matrix(veh),
+                            lkm = lkm,
+                            ef = ef,
+                            month = month,
+                            emis = numeric(nrowv*ncolv*pmonth))$emis
+
+          }
 
           e <- data.frame(emissions = a)
           e <- Emissions(e)
@@ -585,6 +807,7 @@ emis_hot_td <- function (veh,
   } else {
     if(verbose) message("Estimation without monthly profile")
 
+    # !is.data.frame(ef) ####
     if(!is.data.frame(ef)) {
 
       if(verbose) message("'ef' is a numeric vector with units")
@@ -595,15 +818,38 @@ emis_hot_td <- function (veh,
         nrowv = as.integer(nrow(veh))
         ncolv = as.integer(ncol(veh))
 
-        if(verbose) message("Calling emistd7f.f95")
+        if(length(ef) != ncol(veh)) stop("length of `ef` and number of cols of `veh` must be equal")
+        if(length(lkm) != ncol(veh)) stop("length of `lkm` must be equal to number of columns of `veh`")
+        # emis(i, j) = veh(i,j) * lkm(j) * ef(j)
 
-        a <-   .Fortran("emistd7f",
-                        nrowv = nrowv,
-                        ncolv = ncolv,
-                        veh = as.matrix(veh),
-                        lkm = lkm,
-                        ef = ef,
-                        emis = numeric(nrowv*ncolv))$emis
+
+        if(!missing(nt)) {
+          if(nt >= check_nt()) stop("Your machine has ", check_nt(),
+                                    " threads and nt must be lower")
+
+          if(verbose) message("Calling emistd7fpar.f95")
+
+          a <-   .Fortran("emistd7fpar",
+                          nrowv = nrowv,
+                          ncolv = ncolv,
+                          veh = as.matrix(veh),
+                          lkm = lkm,
+                          ef = ef,
+                          emis = numeric(nrowv*ncolv),
+                          nt = as.integer(nt))$emis
+
+        } else {
+          if(verbose) message("Calling emistd7f.f95")
+
+          a <-   .Fortran("emistd7f",
+                          nrowv = nrowv,
+                          ncolv = ncolv,
+                          veh = as.matrix(veh),
+                          lkm = lkm,
+                          ef = ef,
+                          emis = numeric(nrowv*ncolv))$emis
+
+        }
         # fortran
         # do concurrent(i= 1:nrowv, j = 1:ncolv)
         # emis(i, j) = veh(i,j) * lkm(i) * ef(j)
@@ -625,6 +871,7 @@ emis_hot_td <- function (veh,
         e$age <- rep(1:ncol(veh), each = nrow(veh))
 
       }
+      # ef is data.frame
     } else {
       if(verbose) message("'ef' is data.frame")
       if(nrow(ef) != nrow(veh)) stop("Number of rows of 'ef' and 'veh' must be equal")
@@ -649,6 +896,7 @@ emis_hot_td <- function (veh,
         e[, names(params)[i]] <- params[[i]]
       }
     }
+
     if(verbose) cat("Sum of emissions:", sum(e$emissions), "\n")
 
   }
